module searchblock (CLK, din, addr, ram_we, ss_we, tbl_we, result, complete, reset);


input CLK;
input [15:0] din, addr; //input data bus
input ram_we; //database write enable
input ss_we;  //search string write enable
input tbl_we; //table write enable
input reset;  //hold off searching until low


output [15:0] result; //address of match 
output complete;      //searching is done

wire[7:0] ss_output, tbl_output, db_output;


reg complete;

reg [7:0] dbi, ssj;

reg [3:0] searchState;
reg [15:0] result;

//reg [15:0] i,j;

reg [15:0] tbl_addr, ss_addr, db_addr;
parameter init = 0, search = 1, done = 2, ramwait = 3;

//assign complete = (state == done);

always @(posedge CLK)
begin
	//result <= db_output;
	if (reset == 1)
	begin
		searchState <= init;
	end
	else
	begin
	case (searchState)
		init:
		begin
			//i <= 0; //i=0; //beginning of current match in database
			//j <= 0; //j=0; //position of current character in searchString
			db_addr <= 0;
			ss_addr <= 0;
			tbl_addr <= 0;
			searchState <= ramwait;
		end
		search: //preconditions : db_output is db[i], ss_output is ss[j], tbl_output is tbl[j]
		begin
			if (ss_output == 0 && ss_addr[7:0] != 8'hff)
			begin
				searchState <= done;
				result <= db_addr - ss_addr;
				complete <= 1;
			end
			else if (db_output == 0)
			begin
				searchState <= done;
				result <= -1;
				complete <= 1;
			end
			else if (ss_addr[7:0] == 8'hff || db_output == ss_output)
			begin
				//i <= i + 1;
				//j <= j + 1;
				
				db_addr <= db_addr + 1;
				ss_addr <= ss_addr + 1;
				tbl_addr <= ss_addr + 1;
				searchState <= ramwait;
			end
			else
			begin
				tbl_addr <= tbl_output;
				ss_addr <= tbl_output;
				searchState <= ramwait;
			end
		end
		ramwait:
		begin
			searchState <= search;
		end
		done:
		begin
			searchState <= done;
		end
	endcase
	end
end


ram_infer table_sram(tbl_output, (tbl_we ? addr : tbl_addr), din, tbl_we, CLK);
ram_infer searchString_sram(ss_output, (ss_we ? addr : ss_addr), din, ss_we, CLK);
ram_infer database_sram(db_output, (ram_we ? addr : db_addr), din, ram_we, CLK);

endmodule


// data memory: 
// single read and write
// taken from Altera HDL style manual page 6-17
// Synchronous RAM with Read-Through-Write Behavior
// and modified for 8 bit access
// of 256 bytes
module ram_infer (q, a, d, we, clk);
output  [7:0] q;
input [7:0] d;
input [7:0] a;
input we, clk;
reg [7:0] read_add;
reg [7:0] mem [255:0];
	always @ (negedge clk) begin
	if (we) mem[a] <= d;
	read_add <= a;
	end
	assign q = mem[read_add];
endmodule 
